open GL
open Glu
open Glut

let b_down = ref false
let anglex = ref (-46)
let angley = ref (-32)
let xold = ref 0
let yold = ref 0
let list_id = ref 0

(* {{{ setShaders *)

let setShaders() =
  let v = glCreateShader GL_VERTEX_SHADER in
  let f = glCreateShader GL_FRAGMENT_SHADER in

  (*
  let vs = "
void main()
{
        // Transforming The Vertex
        gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
}
  " in
  let fs = "
Void main()
{
        // Setting Each Pixel To Red
        gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);
}
  " in
  *)
  let vs, fs = "
varying vec3 normal;
varying vec3 vertex_to_light_vector;

void main()
{
        // Transforming The Vertex
        gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;

        // Transforming The Normal To ModelView-Space
        normal = gl_NormalMatrix * gl_Normal; 

        // Transforming The Vertex Position To ModelView-Space
        vec4 vertex_in_modelview_space = gl_ModelViewMatrix * gl_Vertex;

        // Calculating The Vector From The Vertex Position To The Light Position
        vertex_to_light_vector = vec3(gl_LightSource[0].position - vertex_in_modelview_space);
}
  ", "
varying vec3 normal;
varying vec3 vertex_to_light_vector;

void main()
{
        // Defining The Material Colors
        const vec4 AmbientColor = vec4(0.1, 0.0, 0.0, 1.0);
        const vec4 DiffuseColor = vec4(1.0, 0.0, 0.0, 1.0);

        // Scaling The Input Vector To Length 1
        vec3 normalized_normal = normalize(normal);
        vec3 normalized_vertex_to_light_vector = normalize(vertex_to_light_vector);

        // Calculating The Diffuse Term And Clamping It To [0;1]
        float DiffuseTerm = clamp(dot(normal, vertex_to_light_vector), 0.0, 1.0);

        // Calculating The Final Color
        gl_FragColor = AmbientColor + DiffuseColor * DiffuseTerm;
}
  " in

  glShaderSource v vs;
  glShaderSource f fs;

  glCompileShader v;
  glCompileShader f;

  let p = glCreateProgram() in
  
  glAttachShader p v;
  glAttachShader p f;

  glLinkProgram p;
  glUseProgram p;

  print_endline(glGetShaderInfoLog v);
  print_endline(glGetShaderInfoLog f);
  print_endline(glGetProgramInfoLog p);
;;

(* }}} *)
(* {{{ initialize lights *)
let init_lights() =
  glEnable GL_LIGHTING;

  (* set up light colors (ambient, diffuse, specular) *)
  let lightKa = (0.2, 0.2, 0.2, 1.0)    (* ambient light *)
  and lightKd = (0.7, 0.7, 0.7, 1.0)    (* diffuse light *)
  and lightKs = (1.0, 1.0, 1.0, 1.0)    (* specular light *)
  in
  glLight (GL_LIGHT 0) (Light.GL_AMBIENT  lightKa);
  glLight (GL_LIGHT 0) (Light.GL_DIFFUSE  lightKd);
  glLight (GL_LIGHT 0) (Light.GL_SPECULAR lightKs);

  (* position the light *)
  let lightPos = (0., 0., 20., 1.) in   (* positional light *)
  glLight (GL_LIGHT 0) (Light.GL_POSITION lightPos);

  glEnable GL_LIGHT0;                       (* MUST enable each light source after configuration *)
;;
(* }}} *)
(* {{{ callback display *)

let display () =
  glClear ~mask:[GL_COLOR_BUFFER_BIT; GL_DEPTH_BUFFER_BIT];

  glLoadIdentity();
  glTranslate ~x:(0.) ~y:(0.) ~z:(-12.0);

  glRotate ~angle:(float(- !angley)) ~x:1.0 ~y:0.0 ~z:0.0;
  glRotate ~angle:(float(- !anglex)) ~x:0.0 ~y:1.0 ~z:0.0;

  glutSolidSphere ~radius:2.0 ~slices:12 ~stacks:10;

  glFlush();
  glutSwapBuffers();
;;
(* }}} *)
(* {{{ callback reshape *)

let reshape ~width ~height =
  glMatrixMode GL_PROJECTION;
  glLoadIdentity();
  gluPerspective 30. (float width /. float height) 2. 30.;
  glViewport 0 0 width height;
  glMatrixMode GL_MODELVIEW;
  glutPostRedisplay();
;;
(* }}} *)
(* {{{ callback keyboard *)

let keyboard ~key ~x ~y =
  match key with
  | 'e' -> Printf.printf " %d %d\n%!" !anglex !angley;
  | 'p' -> glPolygonMode GL_FRONT_AND_BACK  GL_FILL;  glutPostRedisplay();
  | 'f' -> glPolygonMode GL_FRONT_AND_BACK  GL_LINE;  glutPostRedisplay();
  | 's' -> glPolygonMode GL_FRONT_AND_BACK  GL_POINT; glutPostRedisplay();
  | 'd' -> glEnable GL_DEPTH_TEST; glutPostRedisplay();
  | 'D' -> glDisable GL_DEPTH_TEST; glutPostRedisplay();
  | '\027'
  | 'q' -> exit(0)
  | _ -> ()
;;
(* }}} *)
(* {{{ callback mouse *)

let mouse ~button ~state ~x ~y =
  match button, state with
  | GLUT_LEFT_BUTTON, GLUT_DOWN ->
      b_down := true;
      xold := x;
      yold := y;
  | GLUT_LEFT_BUTTON, GLUT_UP ->
      b_down := false;
  | _ -> ()
;;
(* }}} *)
(* {{{ callback motion *)

let motion ~x ~y =
  if !b_down then  (* if the left button is down *)
  begin
 (* change the rotation angles according to the last position
    of the mouse and the new one *)
    anglex := !anglex + (!xold - x);
    angley := !angley + (!yold - y);
    glutPostRedisplay();
  end;
  
  xold := x;
  yold := y;
;;
(* }}} *)
(* {{{ main and init of GL & Glut *)

let () =
  ignore(glutInit Sys.argv);

  glutInitDisplayMode[GLUT_RGBA; GLUT_DOUBLE; GLUT_DEPTH];
  glutInitWindowPosition ~x:200 ~y:200;
  glutInitWindowSize ~width:640 ~height:480;

  ignore(glutCreateWindow ~title:"demo");

  glPointSize ~size:2.0;
  glClearColor ~r:0.2 ~g:0.3 ~b:0.5 ~a:0.0;

  (* init openGL *)
  glEnable GL_DEPTH_TEST;

  setShaders();
  init_lights();

  (* callback functions *)
  glutDisplayFunc ~display;
  glutReshapeFunc ~reshape;
  glutKeyboardFunc ~keyboard;
  glutMouseFunc ~mouse;
  glutMotionFunc ~motion;

  (* enter the main loop *)
  glutMainLoop();
;;
(* }}} *)

(* vim: sw=2 sts=2 ts=2 et fdm=marker
 *)
